---
 drivers/pci/probe.c |   34 ++++++++++++++++++++++++++++++++++
 include/linux/pci.h |   23 +++++++++++++++++++++++
 2 files changed, 57 insertions(+)

Index: linux-2.6/include/linux/pci.h
===================================================================
--- linux-2.6.orig/include/linux/pci.h
+++ linux-2.6/include/linux/pci.h
@@ -373,6 +373,29 @@ struct pci_dev {
 struct resource *pci_dev_resource_n(struct pci_dev *dev, int n);
 int pci_dev_resource_idx(struct pci_dev *dev, struct resource *res);
 
+#define PCI_STD_RES		(1<<0)
+#define PCI_ROM_RES		(1<<1)
+#define PCI_IOV_RES		(1<<2)
+#define PCI_BRIDGE_RES		(1<<3)
+
+#define PCI_ALL_RES		(PCI_STD_RES | PCI_ROM_RES | PCI_BRIDGE_RES | PCI_IOV_RES)
+#define PCI_NOSTD_RES		(PCI_ALL_RES & ~PCI_STD_RES)
+#define PCI_NOIOV_RES		(PCI_ALL_RES & ~PCI_IOV_RES)
+#define PCI_NOROM_RES		(PCI_ALL_RES & ~PCI_ROM_RES)
+#define PCI_NOBRIDGE_RES	(PCI_ALL_RES & ~PCI_BRIDGE_RES)
+#define PCI_STD_ROM_RES		(PCI_STD_RES | PCI_ROM_RES)
+#define PCI_STD_IOV_RES		(PCI_STD_RES | PCI_IOV_RES)
+#define PCI_STD_ROM_IOV_RES	(PCI_STD_RES | PCI_ROM_RES | PCI_IOV_RES)
+
+int pci_next_resource_idx(int i, int flag);
+
+#define for_each_pci_resource(dev, res, i, flag)	\
+	for (i = pci_next_resource_idx(-1, flag),	\
+		res = pci_dev_resource_n(dev, i);	\
+	     res;					\
+	     i = pci_next_resource_idx(i, flag),	\
+		res = pci_dev_resource_n(dev, i))
+
 static inline struct pci_dev *pci_physfn(struct pci_dev *dev)
 {
 #ifdef CONFIG_PCI_IOV
Index: linux-2.6/drivers/pci/probe.c
===================================================================
--- linux-2.6.orig/drivers/pci/probe.c
+++ linux-2.6/drivers/pci/probe.c
@@ -124,6 +124,40 @@ int pci_dev_resource_idx(struct pci_dev
 	return -1;
 }
 
+static bool check_resource_idx(int *i, int start, int end, bool match)
+{
+	if (*i >= start && *i <= end) {
+		if (match)
+			return true;
+		else
+			*i = end + 1;
+	}
+	return false;
+}
+
+int pci_next_resource_idx(int i, int flag)
+{
+	while (++i < PCI_NUM_RESOURCES) {
+		if (check_resource_idx(&i, PCI_STD_RESOURCES,
+				PCI_STD_RESOURCE_END, !!(flag & PCI_STD_RES)))
+			return i;
+		if (check_resource_idx(&i, PCI_ROM_RESOURCE,
+				PCI_ROM_RESOURCE, !!(flag & PCI_ROM_RES)))
+			return i;
+#ifdef CONFIG_PCI_IOV
+		if (check_resource_idx(&i, PCI_IOV_RESOURCES,
+				 PCI_IOV_RESOURCE_END, !!(flag & PCI_IOV_RES)))
+			return i;
+#endif
+		if (check_resource_idx(&i, PCI_BRIDGE_RESOURCES,
+				PCI_BRIDGE_RESOURCE_END,
+				!!(flag & PCI_BRIDGE_RES)))
+			return i;
+	}
+
+	return -1;
+}
+
 static u64 pci_size(u64 base, u64 maxbase, u64 mask)
 {
 	u64 size = mask & maxbase;	/* Find the significant bits */
